# -*- coding: utf-8 -*-
"""flake8-checkstyle - a flake8 plugin to output Checkstyle XML reports."""

__author__ = """Daichi Kumagai"""
__email__ = 'd9magai@gmail.com'
__version__ = '0.1.0'


from .plugin import CheckstylePlugin

__all__ = (
    'CheckstylePlugin',
)



#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""Tests for the flake8 Checkstyle plugin."""

try:
    from StringIO import StringIO
except ImportError:
    from io import StringIO

import optparse
import sys

from flake8_checkstyle import CheckstylePlugin

from flake8.style_guide import Violation


def options(**kwargs):
    """Create an optparse.Values instance."""
    kwargs.setdefault('output_file', None)
    return optparse.Values(kwargs)


def test_handle():
    """Verify that a formatter will call format from handle."""
    formatter = CheckstylePlugin(options(show_source=False))
    error = Violation(
        code='A001',
        filename='example.py',
        line_number=1,
        column_number=1,
        text='Fake error',
        physical_line='a = 1',
    )

    formatter.handle(error)
    formatter.finished('example.py')
    io = StringIO()
    sys.stdout = io
    formatter.stop()
    sys.stdout = sys.__stdout__
    assert io.getvalue() == '<?xml version=\'1.0\' encoding=\'utf-8\'?>\n<checkstyle><file name="example.py"><error column="1" line="1" message="A001 Fake error" severity="info" source="undefined code" /></file></checkstyle>\n'



#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""Setuptools script for flake8-checkstyle."""

from setuptools import setup

with open('README.rst') as readme_file:
    readme = readme_file.read()

with open('HISTORY.rst') as history_file:
    history = history_file.read()

requirements = [
    'flake8>=3.0.0'
]

test_requirements = [
    'tox',
    'pytest'
]

setup(
    name='flake8-checkstyle',
    version='0.1.0',
    description='Output Checkstyle XML reports of flake8 violations.',
    long_description=readme + '\n\n' + history,
    author='Daichi Kumagai',
    author_email='d9magai@gmail.com',
    url='https://github.com/d9magai/flake8-checkstyle',
    packages=[
        'flake8_checkstyle',
    ],
    package_dir={'flake8_checkstyle': 'flake8_checkstyle'},
    include_package_data=True,
    install_requires=requirements,
    license='MIT License',
    entry_points={
        'flake8.report': [
            'checkstyle = flake8_checkstyle:CheckstylePlugin',
        ]
    },
    zip_safe=False,
    keywords='flake8 checkstyle',
    classifiers=[
        'Development Status :: 4 - Beta',
        'Framework :: Flake8',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: MIT License',
        'Natural Language :: English',
        'Programming Language :: Python :: 2',
        'Programming Language :: Python :: 2.7',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.4',
        'Programming Language :: Python :: 3.5',
        'Programming Language :: Python :: 3.6',
    ],
    test_suite='tests',
    tests_require=test_requirements
)



# -*- coding: utf-8 -*-
"""flake8-checkstyle - a flake8 plugin to output Checkstyle XML reports.

This formatter plugin output XML to stdout.
"""
from io import BytesIO
from xml.etree import cElementTree as ET

from flake8.formatting.base import BaseFormatter


#: The first matching prefix determines the severity(inconsistent: info).
CHECKSTYLE_SERVERITY = {
    'E': 'error',
    'W': 'warning',
}

#: The first matching prefix determines the source
#: (inconsistent: undefined code).
CHECKSTYLE_SOURCE = {
    'E': 'pycodestyle',
    'W': 'pycodestyle',
    'F': 'pyflakes',
    'C': 'mccabe',
    'H': 'hacking',
    'I': 'flake8-import-order',
    'D': 'flake8-docstrings',
}


class CheckstylePlugin(BaseFormatter):
    """A plugin for flake8 to output Checkstyle XML reports."""

    def __init__(self, options):
        """Initialize members."""
        super(CheckstylePlugin, self).__init__(options)
        self.errors = []
        self.checkstyle_element = ET.Element('checkstyle')

    def beginning(self, _):
        """Reset the per-file list of errors."""
        self.errors = []

    def handle(self, error):
        """Add an error to the end of the self.errors."""
        self.errors.append((error))

    def finished(self, filename):
        """Make Checkystyle ElementTree."""
        if len(self.errors) < 1:
            return

        element = ET.SubElement(self.checkstyle_element, 'file', name=filename)
        for error in self.errors:
            message = error.code + ' ' + error.text
            prefix = error.code.strip()[0]
            ET.SubElement(element, 'error', {
                'severity': CHECKSTYLE_SERVERITY.get(prefix, 'info'),
                'line': str(error.line_number),
                'column': str(error.column_number),
                'message': message,
                'source': CHECKSTYLE_SOURCE.get(prefix, 'undefined code'),
            }
            )

    def stop(self):
        """Output Checkstyle XML reports."""
        et = ET.ElementTree(self.checkstyle_element)
        f = BytesIO()
        et.write(f, encoding='utf-8', xml_declaration=True)
        print(f.getvalue().decode('utf-8'))
